@@ -21,9 +21,9 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py 3912 2008-05-21 16:57:40Z richb $"
-__version__   = "$Revision: 3912 $"
-__date__      = "$Date: 2008-05-21 12:57:40 -0400 (Wed, 21 May 2008) $"
+__id__        = "$Id: settings.py 4044 2008-07-18 12:45:13Z wwalker $"
+__version__   = "$Revision: 4044 $"
+__date__      = "$Date: 2008-07-18 08:45:13 -0400 (Fri, 18 Jul 2008) $"
 __copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -80,9 +80,12 @@ try:
 except:
     gconfClient = None
 
+import pyatspi
+
 import debug
 from acss import ACSS
 from orca_i18n import _           # for gettext support
+from orca_i18n import Q_          # to provide qualified translatable strings
 
 # These are the settings that Orca supports the user customizing.
 #
@@ -96,6 +99,7 @@ userCustomizableSettings = [
     "readTableCellRow",
     "enableSpeechIndentation",
     "enableEchoByWord",
+    "enableEchoBySentence",
     "enableKeyEcho",
     "enablePrintableKeys",
     "enableModifierKeys",
@@ -105,6 +109,8 @@ userCustomizableSettings = [
     "enableNavigationKeys",
     "enableBraille",
     "enableBrailleGrouping",
+    "disableBrailleEOL",
+    "brailleEOLIndicator",
     "brailleVerbosityLevel",
     "brailleRolenameStyle",
     "brailleSelectorIndicator",
@@ -114,6 +120,7 @@ userCustomizableSettings = [
     "enableMagLiveUpdating",
     "enableMagCursor",
     "enableMagCursorExplicitSize",
+    "magHideCursor",
     "magCursorSize",
     "magCursorColor",
     "enableMagCrossHair",
@@ -163,7 +170,16 @@ userCustomizableSettings = [
     "enableContractedBraille",
     "brailleContractionTable",
     "enableMouseReview",
-    "mouseDwellDelay"
+    "mouseDwellDelay",
+    "speakCellCoordinates",
+    "speakCellSpan",
+    "speakCellHeaders",
+    "skipBlankCells",
+    "largeObjectTextLength",
+    "wrappedStructuralNavigation",
+    "presentRequiredState",
+    "brailleRequiredStateString",
+    "speechRequiredStateString"
 ]
 
 # The name of the module that hold the user interface for the main window
@@ -329,6 +345,11 @@ enableBrailleGrouping   = False
 #
 enableBrailleMonitor    = False
 
+# string to indicate end of printed line for braille displays:
+#
+disableBrailleEOL = False
+brailleEOLIndicator = " $l"
+
 # Strings used to indicate checkbox/radio button states in braille:
 #
 brailleCheckBoxIndicators = ["< >", "<x>", "<->"]
@@ -352,6 +373,10 @@ enableMagCursor                  = True
 #
 enableMagCursorExplicitSize      = False
 
+# If True, hide the system cursor.
+#
+magHideCursor                    = False
+
 # Size of the magnification cursor (in pixels).
 #
 magCursorSize                    = 32
@@ -501,6 +526,12 @@ magTargetDisplay                 = ''
 #
 enableEchoByWord        = False
 
+# if True, enable word echo.
+# Note that it is allowable for both enableEchoByWord and enableEchoBySentence
+# to be True.
+#
+enableEchoBySentence    = False
+
 # If True, enable key echo.
 # Note that it is allowable for both enableEchoByWord and enableKeyEcho
 # to be True
@@ -568,6 +599,26 @@ enableProgressBarUpdates = False
 #
 progressBarUpdateInterval = 10
 
+# Whether or not to present the 'read only' attribute of text areas
+# if we can detect they are read only or not.
+#
+presentReadOnlyText = True
+
+# Translators: this is used to indicate the user is in a text
+# area that is not editable.  It is meant to be spoken to the user.
+#
+# ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+#
+speechReadOnlyString = Q_("text|read only")
+
+# Translators: this is used to indicate the user is in a text
+# area that is not editable.  It is meant to be a short abbreviation
+# to be presented on the braille display.
+#
+# ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+#
+brailleReadOnlyString = Q_("text|rdonly")
+
 # The complete list of possible text attributes.
 #
 allTextAttributes = \
@@ -697,6 +748,14 @@ useBonoboMain           = True
 #
 asyncMode               = True
 
+# A list of toolkits whose events we need to process synchronously.
+# The only one right now is the Java toolkit (see bug #531869), but
+# we put this here to allow more toolkits to be more easily added
+# and for Java to be removed if issues with asynchronous processing
+# of its events are ever resolved.
+#
+synchronousToolkits     = ["J2SE-access-bridge"]
+
 # If True, we output debug information for the event queue.  We
 # use this in addition to log level to prevent debug logic from
 # bogging down event handling.
@@ -928,6 +987,12 @@ setScriptMapping(re.compile(_('Bon Echo')), "Mozilla")
 #
 setScriptMapping(re.compile(_('Minefield')), "Mozilla")
 
+# Translators: see the regular expression note above.  This is for a
+# version of Thunderbird, which chooses to now call itself by a different
+# name.
+#
+setScriptMapping(re.compile(_('Shredder')), "Thunderbird")
+
 # Translators: see the regular expression note above.  This is for
 # the Thunderbird e-mail application.
 #
@@ -985,6 +1050,33 @@ brailleContractionTable = ''
 # 
 useCollection = True
 
+# Whether or not to speak the cell's coordinates when navigating
+# from cell to cell in a table.
+#
+speakCellCoordinates = True
+
+# Whether or not to speak the number of cells spanned by a cell
+# that occupies more than one row or column of a table.
+#
+speakCellSpan = True
+
+# Whether or not to announce the header that applies to the current
+# when navigating from cell to cell in a table.
+#
+speakCellHeaders = True
+
+# Whether blank cells should be skipped when navigating in a table
+# using table navigation commands.
+#
+skipBlankCells = False
+
+# The minimum size in characters to be considered a "large object"
+# or "chunk" for structural navigation.
+#
+largeObjectTextLength = 75
+
+# Whether to wrap around the document when structural navigation is used.
+wrappedStructuralNavigation = True
 
 # Report object under mouse.
 #
@@ -998,3 +1090,68 @@ mouseDwellDelay = 0
 # Maximum allowed drift while pointer is dwelling in mouse review mode.
 # 
 mouseDwellMaxDrift = 3
+
+# The different modifiers/modifier masks associated with keybindings
+#
+NO_MODIFIER_MASK              =  0
+ALT_MODIFIER_MASK             =  1 << pyatspi.MODIFIER_ALT
+CTRL_MODIFIER_MASK            =  1 << pyatspi.MODIFIER_CONTROL
+ORCA_MODIFIER_MASK            =  1 << MODIFIER_ORCA
+ORCA_ALT_MODIFIER_MASK        = (1 << MODIFIER_ORCA |
+                                 1 << pyatspi.MODIFIER_ALT)
+ORCA_CTRL_MODIFIER_MASK       = (1 << MODIFIER_ORCA |
+                                 1 << pyatspi.MODIFIER_CONTROL)
+ORCA_CTRL_ALT_MODIFIER_MASK   = (1 << MODIFIER_ORCA |
+                                 1 << pyatspi.MODIFIER_CONTROL |
+                                 1 << pyatspi.MODIFIER_ALT)
+ORCA_SHIFT_MODIFIER_MASK      = (1 << MODIFIER_ORCA |
+                                 1 << pyatspi.MODIFIER_SHIFT)
+SHIFT_MODIFIER_MASK           =  1 << pyatspi.MODIFIER_SHIFT
+SHIFT_ALT_MODIFIER_MASK       = (1 << pyatspi.MODIFIER_SHIFT |
+                                 1 << pyatspi.MODIFIER_ALT)
+COMMAND_MODIFIER_MASK         = (1 << pyatspi.MODIFIER_ALT |
+                                 1 << pyatspi.MODIFIER_CONTROL |
+                                 1 << pyatspi.MODIFIER_META2 |
+                                 1 << pyatspi.MODIFIER_META3)
+NON_LOCKING_MODIFIER_MASK     = (1 << pyatspi.MODIFIER_SHIFT |
+                                 1 << pyatspi.MODIFIER_ALT |
+                                 1 << pyatspi.MODIFIER_CONTROL |
+                                 1 << pyatspi.MODIFIER_META2 |
+                                 1 << pyatspi.MODIFIER_META3 |
+                                 1 << MODIFIER_ORCA)
+ALL_BUT_NUMLOCK_MODIFIER_MASK = (1 << MODIFIER_ORCA |
+                                 1 << pyatspi.MODIFIER_SHIFT |
+                                 1 << pyatspi.MODIFIER_SHIFTLOCK |
+                                 1 << pyatspi.MODIFIER_CONTROL |
+                                 1 << pyatspi.MODIFIER_ALT |
+                                 1 << pyatspi.MODIFIER_META2 |
+                                 1 << pyatspi.MODIFIER_META3)
+
+# The 2nd parameter used when creating a keybinding refers to the
+# modifiers "we care about."  We typically care about all of the
+# modifiers which are not locking keys because we want to avoid
+# conflicts with other commands, such as Orca's command for moving
+# among headings (H) and the Help menu (Alt+H).
+#
+defaultModifierMask = NON_LOCKING_MODIFIER_MASK
+
+# Whether or not we should present objects with STATE_REQUIRED to
+# the user. Currently, this is only seen with ARIA widgets.
+#
+presentRequiredState = False
+
+# Translators: Certain objects (such as form controls on web pages)
+# can have STATE_REQUIRED set on them to inform the user that this
+# field must be filled out. This string is the default string which
+# will be spoken and displayed in braille to indicate this state is
+# present.
+#
+brailleRequiredStateString = _("required")
+
+# Translators: Certain objects (such as form controls on web pages)
+# can have STATE_REQUIRED set on them to inform the user that this
+# field must be filled out. This string is the default string which
+# will be spoken and displayed in braille to indicate this state is
+# present.
+#
+speechRequiredStateString = _("required")
