@@ -19,9 +19,9 @@
 
 """Speaks information about the current object of interest."""
 
-__id__        = "$Id: where_am_I.py 3882 2008-05-07 18:22:10Z richb $"
-__version__   = "$Revision: 3882 $"
-__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__id__        = "$Id: where_am_I.py 4066 2008-07-26 23:35:05Z wwalker $"
+__version__   = "$Revision: 4066 $"
+__date__      = "$Date: 2008-07-26 19:35:05 -0400 (Sat, 26 Jul 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -31,6 +31,7 @@ import orca_state
 import rolenames
 import settings
 import speech
+import text_attribute_names
 import urlparse, urllib2
 
 from orca_i18n import _ # for gettext support
@@ -117,7 +118,8 @@ class WhereAmI:
         elif role == pyatspi.ROLE_PAGE_TAB:
             self._speakPageTab(obj, basicOnly)
 
-        elif role in [pyatspi.ROLE_TEXT,
+        elif role in [pyatspi.ROLE_ENTRY,
+                      pyatspi.ROLE_TEXT,
                       pyatspi.ROLE_TERMINAL]:
             self._speakText(obj, basicOnly)
 
@@ -177,6 +179,10 @@ class WhereAmI:
         accelerator = self._getObjAccelerator(obj)
         utterances.append(accelerator)
 
+        text = self._getRequiredState(obj)
+        if text:
+            utterances.append(text)
+
         debug.println(self._debugLevel, "check box utterances=%s" \
                       % utterances)
         speech.speakUtterances(utterances)
@@ -197,6 +203,10 @@ class WhereAmI:
         text = self._getGroupLabel(obj)
         utterances.append(text)
 
+        text = self._getRequiredState(obj)
+        if text:
+            utterances.append(text)
+
         text = self.getObjLabelAndName(obj) + " " + \
                rolenames.getSpeechForRoleName(obj)
         utterances.append(text)
@@ -273,20 +283,25 @@ class WhereAmI:
         """
 
         utterances = []
-        text = self._getObjLabel(obj)
-        utterances.append(text)
+        label = self._getObjLabel(obj)
+        utterances.append(label)
 
         text = rolenames.getSpeechForRoleName(obj)
         utterances.append(text)
 
         name = self._getObjName(obj)
-        utterances.append(name)
+        if name != label:
+            utterances.append(name)
         
         utterances.extend(self._getSpeechForAllTextSelection(obj))
 
         text = self._getObjAccelerator(obj)
         utterances.append(text)
 
+        text = self._getRequiredState(obj)
+        if text:
+            utterances.append(text)
+
         debug.println(self._debugLevel, "spin button utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -341,6 +356,10 @@ class WhereAmI:
         text = self._getObjAccelerator(obj)
         utterances.append(text)
 
+        text = self._getRequiredState(obj)
+        if text:
+            utterances.append(text)
+
         debug.println(self._debugLevel, "slider utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -466,13 +485,25 @@ class WhereAmI:
         cell = self._script.getAncestor(obj,
                                         [pyatspi.ROLE_TABLE_CELL],
                                         [pyatspi.ROLE_FRAME])
-        if cell:
-            return self._speakTableCell(cell, basicOnly)
+        if cell and not self._script.isLayoutOnly(cell.parent):
+            # [[[TODO: WDW - we handle ROLE_ENTRY specially here because
+            # there is a bug in getRealActiveDescendant: it doesn't dive
+            # deep enough into the hierarchy (see comment #12 of bug
+            # #542714).  So, we'll do this nasty hack until we can feel
+            # more comfortable with mucking around with
+            # getRealActiveDescendant.]]]
+            #
+            if obj.getRole() != pyatspi.ROLE_ENTRY:
+                return self._speakTableCell(cell, basicOnly)
 
         utterances = []
         text = self._getObjLabel(obj)
         utterances.append(text)
 
+        if settings.presentReadOnlyText \
+           and self._script.isReadOnlyTextArea(obj):
+            utterances.append(settings.speechReadOnlyString)
+
         text = rolenames.getSpeechForRoleName(obj)
         utterances.append(text)
 
@@ -545,13 +576,14 @@ class WhereAmI:
             nColumns = 0
         else:
             nColumns = table.nColumns
-            column = table.getColumnAtIndex(obj.getIndexInParent())
+            index = self._script.getCellIndex(obj)
+            column = table.getColumnAtIndex(index)
             header = table.getColumnHeader(column)
             if header:
                 text = self._getObjName(header)
                 utterances.append(text)
 
-            row = table.getRowAtIndex(obj.getIndexInParent())
+            row = table.getRowAtIndex(index)
             header = table.getRowHeader(row)
             if header:
                 text = self._getObjName(header)
@@ -564,6 +596,10 @@ class WhereAmI:
             text = self._getTableCell(obj)
             utterances.append(text)
 
+        text = self._getRequiredState(obj)
+        if text:
+            utterances.append(text)
+
         debug.println(self._debugLevel, "first table cell utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -980,7 +1016,7 @@ class WhereAmI:
             name = obj.description
 
         if name and name != "None":
-            text = name
+            text = name.strip()
         debug.println(self._debugLevel, "%s name=<%s>" % (obj.getRole(), text))
 
         return text
@@ -993,7 +1029,7 @@ class WhereAmI:
         label = self._script.getDisplayedLabel(obj)
 
         if label and label != "None":
-            text = label
+            text = label.strip()
 
         debug.println(self._debugLevel, "%s label=<%s>" % (obj.getRole(), text))
 
@@ -1189,8 +1225,8 @@ class WhereAmI:
             debug.println(self._debugLevel, "??? parent=%s" % parent.getRole())
             return []
         else:
-            row = table.getRowAtIndex(obj.getIndexInParent())
-
+            index = self._script.getCellIndex(obj)
+            row = table.getRowAtIndex(index)
             for i in range(0, table.nColumns):
                 acc = table.getAccessibleAt(row, i)
                 utterances.append(self._getTableCell(acc))
@@ -1267,7 +1303,7 @@ class WhereAmI:
 
         return text
 
-    def _getTextSelection(self, obj):
+    def getTextSelection(self, obj):
         """Get the text selection for the given object.
 
         Arguments:
@@ -1284,12 +1320,12 @@ class WhereAmI:
             [startOffset, endOffset] = textObj.getSelection(i)
 
             debug.println(self._debugLevel,
-                "_getTextSelection: selection start=%d, end=%d" % \
+                "getTextSelection: selection start=%d, end=%d" % \
                 (startOffset, endOffset))
 
             selectedText = textObj.getText(startOffset, endOffset)
             debug.println(self._debugLevel,
-                "_getTextSelection: selected text=<%s>" % selectedText)
+                "getTextSelection: selected text=<%s>" % selectedText)
 
             if i > 0:
                 textContents += " "
@@ -1317,7 +1353,7 @@ class WhereAmI:
         text = obj.queryText()
         if text.getNSelections() > 0:
             [textContents, startOffset, endOffset] = \
-                                            self._getTextSelection(obj)
+                                            self.getTextSelection(obj)
 
         if not basicOnly:
             current = obj
@@ -1331,7 +1367,7 @@ class WhereAmI:
                         prevObjText = prevObj.queryText()
                         if prevObjText.getNSelections() > 0:
                             [newTextContents, start, end] = \
-                                         self._getTextSelection(prevObj)
+                                         self.getTextSelection(prevObj)
                             textContents = newTextContents + " " + textContents
                             current = prevObj
                             morePossibleSelections = True
@@ -1353,7 +1389,7 @@ class WhereAmI:
                         nextObjText = nextObj.queryText()
                         if nextObjText.getNSelections() > 0:
                             [newTextContents, start, end] = \
-                                         self._getTextSelection(nextObj)
+                                         self.getTextSelection(nextObj)
                             textContents += " " + newTextContents
                             current = nextObj
                             morePossibleSelections = True
@@ -1535,8 +1571,11 @@ class WhereAmI:
                           "charDict: %s" % (charDict))
 
             for key in keys:
-                if charDict.has_key(key):
+                localizedKey = text_attribute_names.getTextAttributeName(key)
+                if key in charDict:
                     attribute = charDict[key]
+                    localizedValue = \
+                        text_attribute_names.getTextAttributeName(attribute)
                     if attribute:
                         # If it's the 'weight' attribute and greater than 400,
                         # just speak it as bold, otherwise speak the weight.
@@ -1550,15 +1589,15 @@ class WhereAmI:
                         elif key == "underline":
                             if attribute != "none":
                                 attribStr += " "
-                                attribStr += key
+                                attribStr += localizedKey
 
                         elif key == "style":
                             if attribute != "normal":
                                 attribStr += " "
-                                attribStr += attribute
+                                attribStr += localizedValue
                         else:
                             attribStr += " "
-                            attribStr += (key + " " + attribute)
+                            attribStr += (localizedKey + " " + localizedValue)
 
             # Also check to see if this is a hypertext link.
             #
@@ -1603,7 +1642,9 @@ class WhereAmI:
         1. The contents of the title bar of the application main window
         2. If in a dialog box within an application, the contents of the
         title bar of the dialog box.
-        3. Orca will pause briefly between these two pieces of information
+        3. '<n> unfocused dialogs' if this application has more than
+           one unfocused alert or dialog window.
+        4. Orca will pause briefly between these two pieces of information
         so that the speech user can distinguish each.
         """
 
@@ -1618,6 +1659,17 @@ class WhereAmI:
             text = self.getObjLabelAndName(results[1])
             utterances.append(text)
 
+        alertAndDialogCount = \
+                    self._script.getUnfocusedAlertAndDialogCount(obj)
+        if alertAndDialogCount > 0:
+            # Translators: this tells the user how many unfocused
+            # alert and dialog windows that this application has.
+            #
+            line = ngettext("%d unfocused dialog",
+                            "%d unfocused dialogs",
+                            alertAndDialogCount) % alertAndDialogCount
+            utterances.append(line)
+
         debug.println(self._debugLevel, "titlebar utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -1800,3 +1852,23 @@ class WhereAmI:
         debug.println(self._debugLevel, "toolbar utterances=%s" \
                       % utterances)
         speech.speakUtterances(utterances)
+
+    def _getRequiredState(self, obj):
+        """Returns a string describing the required state of the given
+        object.
+
+        Arguments:
+        - obj: the Accessible object
+        """
+
+        if not settings.presentRequiredState:
+            return ""
+
+        if obj.getRole() == pyatspi.ROLE_RADIO_BUTTON:
+            obj = obj.parent
+
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_REQUIRED):
+            return settings.speechRequiredStateString
+        else:
+            return ""
